from . import JertlParser
from . import JertlLexer
from . import JertlListener

class Parser(JertlParser.JertlParser):
    pass

class Lexer(JertlLexer.JertlLexer):
    pass

class Listener(JertlListener.JertlListener):
    pass
